@@ -1132,18 +1132,20 @@ void regenerate_screen(void)
 #endif
 	editwincols = COLS - margin;
 
-	/* Ensure that firstcolumn is the starting column of its chunk. */
-	ensure_firstcolumn_is_aligned();
-
 	/* Do as the website suggests: leave and immediately reenter curses mode. */
 	endwin();
 	doupdate();
 
-	/* Put the terminal in the desired state again, recreate the subwindows
-	 * with their (new) sizes, and redraw the contents of these windows. */
+	/* Put the terminal in the desired state again, and
+	 * recreate the subwindows with their (new) sizes. */
 	terminal_init();
 	window_init();
-	total_refresh();
+
+	/* If we have an open buffer, redraw the contents of the subwindows. */
+	if (openfile) {
+		ensure_firstcolumn_is_aligned();
+		total_refresh();
+	}
 }
 
 /* Handle the global toggle specified in flag. */
@@ -2548,7 +2550,7 @@ int main(int argc, char **argv)
 		if (currmenu != MMAIN)
 			bottombars(MMAIN);
 
-		lastmessage = HUSH;
+		lastmessage = VACUUM;
 		as_an_at = TRUE;
 
 		/* Update the displayed current cursor position only when there
